@@ -36,6 +36,7 @@
 #include <libempathy/empathy-individual-manager.h>
 #include <libempathy/empathy-chatroom-manager.h>
 #include <libempathy/empathy-utils.h>
+#include <libempathy/empathy-contact-list.h>
 
 #include "empathy-account-selector-dialog.h"
 #include "empathy-individual-menu.h"
@@ -43,12 +44,14 @@
 #include "empathy-log-window.h"
 #include "empathy-contact-dialogs.h"
 #include "empathy-gtk-enum-types.h"
+#include "empathy-individual-dialogs.h"
 #include "empathy-individual-edit-dialog.h"
 #include "empathy-individual-information-dialog.h"
 #include "empathy-ui-utils.h"
 #include "empathy-share-my-desktop.h"
 #include "empathy-linking-dialog.h"
 #include "empathy-call-utils.h"
+#include "empathy-individual-store-channel.h"
 
 #define DEBUG_FLAG EMPATHY_DEBUG_CONTACT
 #include <libempathy/empathy-debug.h>
@@ -58,11 +61,13 @@
 typedef struct {
   FolksIndividual *individual; /* owned */
   EmpathyIndividualFeatureFlags features;
+  EmpathyIndividualStore *store; /* may be NULL */
 } EmpathyIndividualMenuPriv;
 
 enum {
   PROP_INDIVIDUAL = 1,
   PROP_FEATURES,
+  PROP_STORE,
 };
 
 enum {
@@ -74,6 +79,33 @@ static guint signals[LAST_SIGNAL];
 
 G_DEFINE_TYPE (EmpathyIndividualMenu, empathy_individual_menu, GTK_TYPE_MENU);
 
+static GtkWidget * empathy_individual_chat_menu_item_new (
+    FolksIndividual *individual);
+static GtkWidget * empathy_individual_sms_menu_item_new (
+    FolksIndividual *individual);
+static GtkWidget * empathy_individual_log_menu_item_new  (
+    FolksIndividual *individual);
+static GtkWidget * empathy_individual_info_menu_item_new (
+    FolksIndividual *individual);
+static GtkWidget * empathy_individual_edit_menu_item_new (
+    FolksIndividual *individual);
+static GtkWidget * empathy_individual_link_menu_item_new (
+    FolksIndividual *individual);
+static GtkWidget * empathy_individual_invite_menu_item_new (
+    FolksIndividual *individual,
+    EmpathyContact *contact);
+static GtkWidget * empathy_individual_file_transfer_menu_item_new (
+    FolksIndividual *individual);
+static GtkWidget * empathy_individual_share_my_desktop_menu_item_new (
+    FolksIndividual *individual);
+static GtkWidget * empathy_individual_favourite_menu_item_new (
+    FolksIndividual *individual);
+static GtkWidget * empathy_individual_add_menu_item_new (
+    EmpathyIndividualMenu *self,
+    FolksIndividual *individual);
+static GtkWidget * empathy_individiual_block_menu_item_new (
+    FolksIndividual *individual);
+
 static void
 individual_menu_add_personas (GtkMenuShell *menu,
     FolksIndividual *individual,
@@ -126,6 +158,8 @@ individual_menu_add_personas (GtkMenuShell *menu,
       FolksPersonaStore *store;
       const gchar *account;
       GtkWidget *action;
+      /* Individual containing only persona */
+      FolksIndividual *single_individual;
 
       if (!empathy_folks_persona_is_interesting (FOLKS_PERSONA (persona)))
         goto while_finish;
@@ -135,6 +169,8 @@ individual_menu_add_personas (GtkMenuShell *menu,
         goto while_finish;
 
       contact = empathy_contact_dup_from_tp_contact (tp_contact);
+      single_individual = empathy_create_individual_from_tp_contact (
+          tp_contact);
 
       store = folks_persona_get_store (FOLKS_PERSONA (persona));
       account = folks_persona_store_get_display_name (store);
@@ -158,7 +194,7 @@ individual_menu_add_personas (GtkMenuShell *menu,
       /* Chat */
       if (features & EMPATHY_INDIVIDUAL_FEATURE_CHAT)
         {
-          action = empathy_individual_chat_menu_item_new (NULL, contact);
+          action = empathy_individual_chat_menu_item_new (single_individual);
           gtk_menu_shell_append (GTK_MENU_SHELL (contact_submenu), action);
           gtk_widget_show (action);
         }
@@ -166,7 +202,7 @@ individual_menu_add_personas (GtkMenuShell *menu,
       /* SMS */
       if (features & EMPATHY_INDIVIDUAL_FEATURE_SMS)
         {
-          action = empathy_individual_sms_menu_item_new (NULL, contact);
+          action = empathy_individual_sms_menu_item_new (single_individual);
           gtk_menu_shell_append (GTK_MENU_SHELL (contact_submenu), action);
           gtk_widget_show (action);
         }
@@ -174,12 +210,14 @@ individual_menu_add_personas (GtkMenuShell *menu,
       if (features & EMPATHY_INDIVIDUAL_FEATURE_CALL)
         {
           /* Audio Call */
-          action = empathy_individual_audio_call_menu_item_new (NULL, contact);
+          action = empathy_individual_audio_call_menu_item_new (
+              single_individual);
           gtk_menu_shell_append (GTK_MENU_SHELL (contact_submenu), action);
           gtk_widget_show (action);
 
           /* Video Call */
-          action = empathy_individual_video_call_menu_item_new (NULL, contact);
+          action = empathy_individual_video_call_menu_item_new (
+              single_individual);
           gtk_menu_shell_append (GTK_MENU_SHELL (contact_submenu), action);
           gtk_widget_show (action);
         }
@@ -187,7 +225,7 @@ individual_menu_add_personas (GtkMenuShell *menu,
       /* Log */
       if (features & EMPATHY_INDIVIDUAL_FEATURE_LOG)
         {
-          action = empathy_individual_log_menu_item_new (NULL, contact);
+          action = empathy_individual_log_menu_item_new (single_individual);
           gtk_menu_shell_append (GTK_MENU_SHELL (contact_submenu), action);
           gtk_widget_show (action);
         }
@@ -198,21 +236,37 @@ individual_menu_add_personas (GtkMenuShell *menu,
       gtk_widget_show (action);
 
       /* File transfer */
-      action = empathy_individual_file_transfer_menu_item_new (NULL, contact);
+      action = empathy_individual_file_transfer_menu_item_new (
+          single_individual);
       gtk_menu_shell_append (GTK_MENU_SHELL (contact_submenu), action);
       gtk_widget_show (action);
 
       /* Share my desktop */
-      action = empathy_individual_share_my_desktop_menu_item_new (NULL,
-          contact);
+      action = empathy_individual_share_my_desktop_menu_item_new (
+          single_individual);
       gtk_menu_shell_append (GTK_MENU_SHELL (contact_submenu), action);
       gtk_widget_show (action);
 
+      /* Block */
+      if (features & EMPATHY_INDIVIDUAL_FEATURE_BLOCK &&
+          (item = empathy_individiual_block_menu_item_new (single_individual))
+          != NULL) {
+        GtkWidget *sep;
+
+        sep = gtk_separator_menu_item_new ();
+        gtk_menu_shell_append (GTK_MENU_SHELL (contact_submenu), sep);
+        gtk_widget_show (sep);
+
+        gtk_menu_shell_append (GTK_MENU_SHELL (contact_submenu), item);
+        gtk_widget_show (item);
+      }
+
       gtk_menu_shell_append (GTK_MENU_SHELL (menu), contact_item);
       gtk_widget_show (contact_item);
 
       g_free (label);
       g_object_unref (contact);
+      g_object_unref (single_individual);
 
 while_finish:
       g_clear_object (&persona);
@@ -439,6 +493,210 @@ add_phone_numbers (EmpathyIndividualMenu *self)
     }
 }
 
+/* return a list of TpContact supporting the blocking iface */
+static GList *
+get_contacts_supporting_blocking (FolksIndividual *individual)
+{
+  GeeSet *personas;
+  GeeIterator *iter;
+  GList *result = NULL;
+
+  personas = folks_individual_get_personas (individual);
+
+  iter = gee_iterable_iterator (GEE_ITERABLE (personas));
+  while (gee_iterator_next (iter))
+    {
+      TpfPersona *persona = gee_iterator_get (iter);
+      TpContact *contact;
+      TpConnection *conn;
+
+      if (!TPF_IS_PERSONA (persona))
+        continue;
+
+      contact = tpf_persona_get_contact (persona);
+      if (contact == NULL)
+        continue;
+
+      conn = tp_contact_get_connection (contact);
+
+      if (tp_proxy_has_interface_by_id (conn,
+        TP_IFACE_QUARK_CONNECTION_INTERFACE_CONTACT_BLOCKING))
+        result = g_list_prepend (result, contact);
+    }
+
+  g_clear_object (&iter);
+
+  return result;
+}
+
+typedef struct
+{
+  gboolean blocked;
+  GtkWidget *parent;
+} GotAvatarCtx;
+
+static GotAvatarCtx *
+got_avatar_ctx_new (gboolean blocked,
+    GtkWidget *parent)
+{
+  GotAvatarCtx *ctx = g_slice_new0 (GotAvatarCtx);
+
+  ctx->blocked = blocked;
+  ctx->parent = parent != NULL ? g_object_ref (parent) : NULL;
+  return ctx;
+}
+
+static void
+got_avatar_ctx_free (GotAvatarCtx *ctx)
+{
+  g_clear_object (&ctx->parent);
+  g_slice_free (GotAvatarCtx, ctx);
+}
+
+static void
+got_avatar (GObject *source_object,
+    GAsyncResult *result,
+    gpointer user_data)
+{
+  FolksIndividual *individual = FOLKS_INDIVIDUAL (source_object);
+  GotAvatarCtx *ctx = user_data;
+  GdkPixbuf *avatar;
+  GError *error = NULL;
+  gboolean abusive = FALSE;
+  EmpathyIndividualManager *manager;
+
+  avatar = empathy_pixbuf_avatar_from_individual_scaled_finish (individual,
+      result, &error);
+
+  if (error != NULL)
+    {
+      DEBUG ("Could not get avatar: %s", error->message);
+      g_error_free (error);
+    }
+
+  if (ctx->blocked) {
+    /* confirm the user really wishes to block the contact */
+    if (!empathy_block_individual_dialog_show (GTK_WINDOW (ctx->parent),
+          individual, avatar, &abusive))
+      goto out;
+  }
+
+  manager = empathy_individual_manager_dup_singleton ();
+
+  empathy_individual_manager_set_blocked (manager, individual,
+      ctx->blocked, abusive);
+
+  g_object_unref (manager);
+
+out:
+  g_clear_object (&avatar);
+  got_avatar_ctx_free (ctx);
+}
+
+static void
+empathy_individual_block_menu_item_toggled (GtkCheckMenuItem *item,
+    FolksIndividual *individual)
+{
+  GotAvatarCtx *ctx;
+  gboolean blocked;
+  GtkWidget *parent;
+
+  /* @item may be destroyed while the async call is running to get the things
+   * we need from it right now. */
+  blocked = gtk_check_menu_item_get_active (item);
+
+  parent = g_object_get_data (
+    G_OBJECT (gtk_widget_get_parent (GTK_WIDGET (item))),
+    "window");
+
+  ctx = got_avatar_ctx_new (blocked, parent);
+
+  empathy_pixbuf_avatar_from_individual_scaled_async (individual,
+      48, 48, NULL, got_avatar, ctx);
+}
+
+static void
+update_block_menu_item (GtkWidget *item,
+    FolksIndividual *individual)
+{
+  GList *contacts, *l;
+  gboolean is_blocked = TRUE;
+
+  contacts = get_contacts_supporting_blocking (individual);
+
+  if (contacts == NULL)
+    is_blocked = FALSE;
+
+  /* Check the menu item if all his personas are blocked */
+  for (l = contacts; l != NULL; l = g_list_next (l))
+    {
+      TpContact *contact = l->data;
+
+      if (!tp_contact_is_blocked (contact))
+        {
+          is_blocked = FALSE;
+          break;
+        }
+    }
+
+  g_signal_handlers_block_by_func (item,
+      empathy_individual_block_menu_item_toggled, individual);
+
+  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item), is_blocked);
+
+  g_signal_handlers_unblock_by_func (item,
+      empathy_individual_block_menu_item_toggled, individual);
+
+  g_list_free (contacts);
+}
+
+static void
+contact_blocked_changed_cb (TpContact *contact,
+    GParamSpec *spec,
+    GtkWidget *item)
+{
+  FolksIndividual *individual;
+
+  individual = g_object_get_data (G_OBJECT (item), "individual");
+
+  update_block_menu_item (item, individual);
+}
+
+static GtkWidget *
+empathy_individiual_block_menu_item_new (FolksIndividual *individual)
+{
+  GtkWidget *item;
+  GList *contacts, *l;
+
+  contacts = get_contacts_supporting_blocking (individual);
+
+  /* Can't block, no persona supports blocking */
+  if (contacts == NULL)
+    return NULL;
+
+  item = gtk_check_menu_item_new_with_mnemonic (_("_Block Contact"));
+
+  g_object_set_data_full (G_OBJECT (item), "individual",
+      g_object_ref (individual), g_object_unref);
+
+  for (l = contacts; l != NULL; l = g_list_next (l))
+    {
+      TpContact *contact = l->data;
+
+      tp_g_signal_connect_object (contact, "notify::is-blocked",
+          G_CALLBACK (contact_blocked_changed_cb), item, 0);
+    }
+
+  g_signal_connect (item, "toggled",
+      G_CALLBACK (empathy_individual_block_menu_item_toggled), individual);
+
+  update_block_menu_item (item, individual);
+
+  g_list_free (contacts);
+
+  return item;
+}
+
 static void
 constructed (GObject *object)
 {
@@ -454,10 +712,21 @@ constructed (GObject *object)
   individual = priv->individual;
   features = priv->features;
 
+  /* Add contact */
+  if (features & EMPATHY_INDIVIDUAL_FEATURE_ADD_CONTACT)
+    {
+      item = empathy_individual_add_menu_item_new (self, individual);
+      if (item != NULL)
+        {
+          gtk_menu_shell_append (GTK_MENU_SHELL (shell), item);
+          gtk_widget_show (item);
+        }
+    }
+
   /* Chat */
   if (features & EMPATHY_INDIVIDUAL_FEATURE_CHAT)
     {
-      item = empathy_individual_chat_menu_item_new (individual, NULL);
+      item = empathy_individual_chat_menu_item_new (individual);
       if (item != NULL)
         {
           gtk_menu_shell_append (shell, item);
@@ -468,7 +737,7 @@ constructed (GObject *object)
   /* SMS */
   if (features & EMPATHY_INDIVIDUAL_FEATURE_SMS)
     {
-      item = empathy_individual_sms_menu_item_new (individual, NULL);
+      item = empathy_individual_sms_menu_item_new (individual);
       if (item != NULL)
         {
           gtk_menu_shell_append (shell, item);
@@ -479,12 +748,12 @@ constructed (GObject *object)
   if (features & EMPATHY_INDIVIDUAL_FEATURE_CALL)
     {
       /* Audio Call */
-      item = empathy_individual_audio_call_menu_item_new (individual, NULL);
+      item = empathy_individual_audio_call_menu_item_new (individual);
       gtk_menu_shell_append (shell, item);
       gtk_widget_show (item);
 
       /* Video Call */
-      item = empathy_individual_video_call_menu_item_new (individual, NULL);
+      item = empathy_individual_video_call_menu_item_new (individual);
       gtk_menu_shell_append (shell, item);
       gtk_widget_show (item);
     }
@@ -498,14 +767,14 @@ constructed (GObject *object)
   gtk_widget_show (item);
 
   /* File transfer */
-  item = empathy_individual_file_transfer_menu_item_new (individual, NULL);
+  item = empathy_individual_file_transfer_menu_item_new (individual);
   gtk_menu_shell_append (shell, item);
   gtk_widget_show (item);
 
   /* Share my desktop */
   /* FIXME we should add the "Share my desktop" menu item if Vino is
   a registered handler in MC5 */
-  item = empathy_individual_share_my_desktop_menu_item_new (individual, NULL);
+  item = empathy_individual_share_my_desktop_menu_item_new (individual);
   gtk_menu_shell_append (shell, item);
   gtk_widget_show (item);
 
@@ -546,7 +815,7 @@ constructed (GObject *object)
   /* Log */
   if (features & EMPATHY_INDIVIDUAL_FEATURE_LOG)
     {
-      item = empathy_individual_log_menu_item_new (individual, NULL);
+      item = empathy_individual_log_menu_item_new (individual);
       gtk_menu_shell_append (shell, item);
       gtk_widget_show (item);
     }
@@ -566,6 +835,19 @@ constructed (GObject *object)
       gtk_menu_shell_append (shell, item);
       gtk_widget_show (item);
     }
+
+  /* Separator & Block */
+  if (features & EMPATHY_INDIVIDUAL_FEATURE_BLOCK &&
+      (item = empathy_individiual_block_menu_item_new (individual)) != NULL) {
+    GtkWidget *sep;
+
+    sep = gtk_separator_menu_item_new ();
+    gtk_menu_shell_append (shell, sep);
+    gtk_widget_show (sep);
+
+    gtk_menu_shell_append (shell, item);
+    gtk_widget_show (item);
+  }
 }
 
 static void
@@ -586,6 +868,9 @@ get_property (GObject *object,
       case PROP_FEATURES:
         g_value_set_flags (value, priv->features);
         break;
+      case PROP_STORE:
+        g_value_set_object (value, priv->store);
+        break;
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
         break;
@@ -610,6 +895,9 @@ set_property (GObject *object,
       case PROP_FEATURES:
         priv->features = g_value_get_flags (value);
         break;
+      case PROP_STORE:
+        priv->store = g_value_dup_object (value); /* read only */
+        break;
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
         break;
@@ -622,6 +910,7 @@ dispose (GObject *object)
   EmpathyIndividualMenuPriv *priv = GET_PRIV (object);
 
   tp_clear_object (&priv->individual);
+  tp_clear_object (&priv->store);
 
   G_OBJECT_CLASS (empathy_individual_menu_parent_class)->dispose (object);
 }
@@ -661,6 +950,13 @@ empathy_individual_menu_class_init (EmpathyIndividualMenuClass *klass)
           EMPATHY_INDIVIDUAL_FEATURE_NONE,
           G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS));
 
+  g_object_class_install_property (object_class, PROP_STORE,
+      g_param_spec_object ("store",
+          "Store",
+          "The EmpathyIndividualStore to use to get contact owner",
+          EMPATHY_TYPE_INDIVIDUAL_STORE,
+          G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS));
+
   signals[SIGNAL_LINK_CONTACTS_ACTIVATED] =
       g_signal_new ("link-contacts-activated", G_OBJECT_CLASS_TYPE (klass),
           G_SIGNAL_RUN_LAST, 0, NULL, NULL,
@@ -672,14 +968,18 @@ empathy_individual_menu_class_init (EmpathyIndividualMenuClass *klass)
 
 GtkWidget *
 empathy_individual_menu_new (FolksIndividual *individual,
-    EmpathyIndividualFeatureFlags features)
+    EmpathyIndividualFeatureFlags features,
+    EmpathyIndividualStore *store)
 {
   g_return_val_if_fail (FOLKS_IS_INDIVIDUAL (individual), NULL);
+  g_return_val_if_fail (store == NULL ||
+      EMPATHY_IS_INDIVIDUAL_STORE (store), NULL);
   g_return_val_if_fail (features != EMPATHY_INDIVIDUAL_FEATURE_NONE, NULL);
 
   return g_object_new (EMPATHY_TYPE_INDIVIDUAL_MENU,
       "individual", individual,
       "features", features,
+      "store", store,
       NULL);
 }
 
@@ -743,17 +1043,14 @@ empathy_individual_chat_menu_item_activated (GtkMenuItem *item,
   empathy_chat_with_contact (contact, empathy_get_current_action_time ());
 }
 
-GtkWidget *
-empathy_individual_chat_menu_item_new (FolksIndividual *individual,
-    EmpathyContact *contact)
+static GtkWidget *
+empathy_individual_chat_menu_item_new (FolksIndividual *individual)
 {
   GtkWidget *item;
   GtkWidget *image;
 
-  g_return_val_if_fail ((FOLKS_IS_INDIVIDUAL (individual) &&
-      empathy_folks_individual_contains_contact (individual)) ||
-      EMPATHY_IS_CONTACT (contact),
-      NULL);
+  g_return_val_if_fail (FOLKS_IS_INDIVIDUAL (individual) &&
+      empathy_folks_individual_contains_contact (individual), NULL);
 
   item = gtk_image_menu_item_new_with_mnemonic (_("_Chat"));
   image = gtk_image_new_from_icon_name (EMPATHY_IMAGE_MESSAGE,
@@ -761,18 +1058,9 @@ empathy_individual_chat_menu_item_new (FolksIndividual *individual,
   gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
   gtk_widget_show (image);
 
-  if (contact != NULL)
-    {
-      menu_item_set_contact (item, contact,
-          G_CALLBACK (empathy_individual_chat_menu_item_activated),
-          EMPATHY_ACTION_CHAT);
-    }
-  else
-    {
-      menu_item_set_first_contact (item, individual,
-          G_CALLBACK (empathy_individual_chat_menu_item_activated),
-          EMPATHY_ACTION_CHAT);
-    }
+  menu_item_set_first_contact (item, individual,
+      G_CALLBACK (empathy_individual_chat_menu_item_activated),
+      EMPATHY_ACTION_CHAT);
 
   return item;
 }
@@ -790,17 +1078,14 @@ empathy_individual_sms_menu_item_activated (GtkMenuItem *item,
       NULL, NULL);
 }
 
-GtkWidget *
-empathy_individual_sms_menu_item_new (FolksIndividual *individual,
-    EmpathyContact *contact)
+static GtkWidget *
+empathy_individual_sms_menu_item_new (FolksIndividual *individual)
 {
   GtkWidget *item;
   GtkWidget *image;
 
-  g_return_val_if_fail ((FOLKS_IS_INDIVIDUAL (individual) &&
-      empathy_folks_individual_contains_contact (individual)) ||
-      EMPATHY_IS_CONTACT (contact),
-      NULL);
+  g_return_val_if_fail (FOLKS_IS_INDIVIDUAL (individual) &&
+      empathy_folks_individual_contains_contact (individual), NULL);
 
   item = gtk_image_menu_item_new_with_mnemonic (_("_SMS"));
   image = gtk_image_new_from_icon_name (EMPATHY_IMAGE_SMS,
@@ -808,18 +1093,9 @@ empathy_individual_sms_menu_item_new (FolksIndividual *individual,
   gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
   gtk_widget_show (image);
 
-  if (contact != NULL)
-    {
-      menu_item_set_contact (item, contact,
-          G_CALLBACK (empathy_individual_sms_menu_item_activated),
-          EMPATHY_ACTION_SMS);
-    }
-  else
-    {
-      menu_item_set_first_contact (item, individual,
-          G_CALLBACK (empathy_individual_sms_menu_item_activated),
-          EMPATHY_ACTION_SMS);
-    }
+  menu_item_set_first_contact (item, individual,
+      G_CALLBACK (empathy_individual_sms_menu_item_activated),
+      EMPATHY_ACTION_SMS);
 
   return item;
 }
@@ -837,33 +1113,21 @@ empathy_individual_audio_call_menu_item_activated (GtkMenuItem *item,
 }
 
 GtkWidget *
-empathy_individual_audio_call_menu_item_new (FolksIndividual *individual,
-    EmpathyContact *contact)
+empathy_individual_audio_call_menu_item_new (FolksIndividual *individual)
 {
   GtkWidget *item;
   GtkWidget *image;
 
-  g_return_val_if_fail (FOLKS_IS_INDIVIDUAL (individual) ||
-      EMPATHY_IS_CONTACT (contact),
-      NULL);
+  g_return_val_if_fail (FOLKS_IS_INDIVIDUAL (individual), NULL);
 
   item = gtk_image_menu_item_new_with_mnemonic (C_("menu item", "_Audio Call"));
   image = gtk_image_new_from_icon_name (EMPATHY_IMAGE_VOIP, GTK_ICON_SIZE_MENU);
   gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
   gtk_widget_show (image);
 
-  if (contact != NULL)
-    {
-      menu_item_set_contact (item, contact,
-          G_CALLBACK (empathy_individual_audio_call_menu_item_activated),
-          EMPATHY_ACTION_AUDIO_CALL);
-    }
-  else
-    {
-      menu_item_set_first_contact (item, individual,
-          G_CALLBACK (empathy_individual_audio_call_menu_item_activated),
-          EMPATHY_ACTION_AUDIO_CALL);
-    }
+  menu_item_set_first_contact (item, individual,
+      G_CALLBACK (empathy_individual_audio_call_menu_item_activated),
+      EMPATHY_ACTION_AUDIO_CALL);
 
   return item;
 }
@@ -881,16 +1145,13 @@ empathy_individual_video_call_menu_item_activated (GtkMenuItem *item,
 }
 
 GtkWidget *
-empathy_individual_video_call_menu_item_new (FolksIndividual *individual,
-    EmpathyContact *contact)
+empathy_individual_video_call_menu_item_new (FolksIndividual *individual)
 {
   GtkWidget *item;
   GtkWidget *image;
   EmpathyCameraMonitor *monitor;
 
-  g_return_val_if_fail (FOLKS_IS_INDIVIDUAL (individual) ||
-      EMPATHY_IS_CONTACT (contact),
-      NULL);
+  g_return_val_if_fail (FOLKS_IS_INDIVIDUAL (individual), NULL);
 
   item = gtk_image_menu_item_new_with_mnemonic (C_("menu item", "_Video Call"));
   image = gtk_image_new_from_icon_name (EMPATHY_IMAGE_VIDEO_CALL,
@@ -898,18 +1159,9 @@ empathy_individual_video_call_menu_item_new (FolksIndividual *individual,
   gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
   gtk_widget_show (image);
 
-  if (contact != NULL)
-    {
-      menu_item_set_contact (item, contact,
-          G_CALLBACK (empathy_individual_video_call_menu_item_activated),
-          EMPATHY_ACTION_VIDEO_CALL);
-    }
-  else
-    {
-      menu_item_set_first_contact (item, individual,
-          G_CALLBACK (empathy_individual_video_call_menu_item_activated),
-          EMPATHY_ACTION_VIDEO_CALL);
-    }
+  menu_item_set_first_contact (item, individual,
+      G_CALLBACK (empathy_individual_video_call_menu_item_activated),
+      EMPATHY_ACTION_VIDEO_CALL);
 
   /* Only follow available cameras if the contact can do Video calls */
   if (gtk_widget_get_sensitive (item))
@@ -934,34 +1186,22 @@ empathy_individual_log_menu_item_activated (GtkMenuItem *item,
       empathy_contact_get_id (contact), FALSE, NULL);
 }
 
-GtkWidget *
-empathy_individual_log_menu_item_new (FolksIndividual *individual,
-    EmpathyContact *contact)
+static GtkWidget *
+empathy_individual_log_menu_item_new (FolksIndividual *individual)
 {
   GtkWidget *item;
   GtkWidget *image;
 
-  g_return_val_if_fail (FOLKS_IS_INDIVIDUAL (individual) ||
-      EMPATHY_IS_CONTACT (contact),
-      NULL);
+  g_return_val_if_fail (FOLKS_IS_INDIVIDUAL (individual), NULL);
 
   item = gtk_image_menu_item_new_with_mnemonic (_("_Previous Conversations"));
   image = gtk_image_new_from_icon_name (EMPATHY_IMAGE_LOG, GTK_ICON_SIZE_MENU);
   gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
   gtk_widget_show (image);
 
-  if (contact != NULL)
-    {
-      menu_item_set_contact (item, contact,
-          G_CALLBACK (empathy_individual_log_menu_item_activated),
-          EMPATHY_ACTION_VIEW_LOGS);
-    }
-  else
-    {
-      menu_item_set_first_contact (item, individual,
-          G_CALLBACK (empathy_individual_log_menu_item_activated),
-          EMPATHY_ACTION_VIEW_LOGS);
-    }
+  menu_item_set_first_contact (item, individual,
+      G_CALLBACK (empathy_individual_log_menu_item_activated),
+      EMPATHY_ACTION_VIEW_LOGS);
 
   return item;
 }
@@ -975,16 +1215,13 @@ empathy_individual_file_transfer_menu_item_activated (GtkMenuItem *item,
   empathy_send_file_with_file_chooser (contact);
 }
 
-GtkWidget *
-empathy_individual_file_transfer_menu_item_new (FolksIndividual *individual,
-    EmpathyContact *contact)
+static GtkWidget *
+empathy_individual_file_transfer_menu_item_new (FolksIndividual *individual)
 {
   GtkWidget *item;
   GtkWidget *image;
 
-  g_return_val_if_fail (FOLKS_IS_INDIVIDUAL (individual) ||
-      EMPATHY_IS_CONTACT (contact),
-      NULL);
+  g_return_val_if_fail (FOLKS_IS_INDIVIDUAL (individual), NULL);
 
   item = gtk_image_menu_item_new_with_mnemonic (_("Send File"));
   image = gtk_image_new_from_icon_name (EMPATHY_IMAGE_DOCUMENT_SEND,
@@ -992,18 +1229,9 @@ empathy_individual_file_transfer_menu_item_new (FolksIndividual *individual,
   gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
   gtk_widget_show (image);
 
-  if (contact != NULL)
-    {
-      menu_item_set_contact (item, contact,
-          G_CALLBACK (empathy_individual_file_transfer_menu_item_activated),
-          EMPATHY_ACTION_SEND_FILE);
-    }
-  else
-    {
-      menu_item_set_first_contact (item, individual,
-          G_CALLBACK (empathy_individual_file_transfer_menu_item_activated),
-          EMPATHY_ACTION_SEND_FILE);
-    }
+  menu_item_set_first_contact (item, individual,
+      G_CALLBACK (empathy_individual_file_transfer_menu_item_activated),
+      EMPATHY_ACTION_SEND_FILE);
 
   return item;
 }
@@ -1017,34 +1245,22 @@ empathy_individual_share_my_desktop_menu_item_activated (GtkMenuItem *item,
   empathy_share_my_desktop_share_with_contact (contact);
 }
 
-GtkWidget *
-empathy_individual_share_my_desktop_menu_item_new (FolksIndividual *individual,
-    EmpathyContact *contact)
+static GtkWidget *
+empathy_individual_share_my_desktop_menu_item_new (FolksIndividual *individual)
 {
   GtkWidget *item;
   GtkWidget *image;
 
-  g_return_val_if_fail (FOLKS_IS_INDIVIDUAL (individual) ||
-      EMPATHY_IS_CONTACT (contact),
-      NULL);
+  g_return_val_if_fail (FOLKS_IS_INDIVIDUAL (individual), NULL);
 
   item = gtk_image_menu_item_new_with_mnemonic (_("Share My Desktop"));
   image = gtk_image_new_from_icon_name (GTK_STOCK_NETWORK, GTK_ICON_SIZE_MENU);
   gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
   gtk_widget_show (image);
 
-  if (contact != NULL)
-    {
-      menu_item_set_contact (item, contact,
-          G_CALLBACK (empathy_individual_share_my_desktop_menu_item_activated),
-          EMPATHY_ACTION_SHARE_MY_DESKTOP);
-    }
-  else
-    {
-      menu_item_set_first_contact (item, individual,
-          G_CALLBACK (empathy_individual_share_my_desktop_menu_item_activated),
-          EMPATHY_ACTION_SHARE_MY_DESKTOP);
-    }
+  menu_item_set_first_contact (item, individual,
+      G_CALLBACK (empathy_individual_share_my_desktop_menu_item_activated),
+      EMPATHY_ACTION_SHARE_MY_DESKTOP);
 
   return item;
 }
@@ -1058,7 +1274,7 @@ favourite_menu_item_toggled_cb (GtkCheckMenuItem *item,
       gtk_check_menu_item_get_active (item));
 }
 
-GtkWidget *
+static GtkWidget *
 empathy_individual_favourite_menu_item_new (FolksIndividual *individual)
 {
   GtkWidget *item;
@@ -1081,7 +1297,7 @@ individual_info_menu_item_activate_cb (FolksIndividual *individual)
   empathy_individual_information_dialog_show (individual, NULL);
 }
 
-GtkWidget *
+static GtkWidget *
 empathy_individual_info_menu_item_new (FolksIndividual *individual)
 {
   GtkWidget *item;
@@ -1110,7 +1326,7 @@ individual_edit_menu_item_activate_cb (FolksIndividual *individual)
   empathy_individual_edit_dialog_show (individual, NULL);
 }
 
-GtkWidget *
+static GtkWidget *
 empathy_individual_edit_menu_item_new (FolksIndividual *individual)
 {
   EmpathyIndividualManager *manager;
@@ -1155,7 +1371,7 @@ empathy_individual_edit_menu_item_new (FolksIndividual *individual)
   return item;
 }
 
-GtkWidget *
+static GtkWidget *
 empathy_individual_link_menu_item_new (FolksIndividual *individual)
 {
   GtkWidget *item;
@@ -1305,7 +1521,7 @@ create_room_sub_menu (FolksIndividual *individual,
   return item;
 }
 
-GtkWidget *
+static GtkWidget *
 empathy_individual_invite_menu_item_new (FolksIndividual *individual,
     EmpathyContact *contact)
 {
@@ -1427,3 +1643,110 @@ empathy_individual_invite_menu_item_new (FolksIndividual *individual,
 
   return item;
 }
+
+static void
+add_menu_item_activated (GtkMenuItem *item,
+    TpContact *tp_contact)
+{
+  GtkWidget *toplevel;
+  EmpathyContact *contact;
+
+  toplevel = gtk_widget_get_toplevel (GTK_WIDGET (item));
+  if (!gtk_widget_is_toplevel (toplevel) || !GTK_IS_WINDOW (toplevel))
+    toplevel = NULL;
+
+  contact = empathy_contact_dup_from_tp_contact (tp_contact);
+
+  empathy_new_contact_dialog_show_with_contact (GTK_WINDOW (toplevel),
+      contact);
+
+  g_object_unref (contact);
+}
+
+static GtkWidget *
+empathy_individual_add_menu_item_new (EmpathyIndividualMenu *self,
+    FolksIndividual *individual)
+{
+  EmpathyIndividualMenuPriv *priv = GET_PRIV (self);
+  GtkWidget *item, *image;
+  GeeSet *personas;
+  GeeIterator *iter;
+  TpContact *to_add = NULL;
+
+  /* find the first of this Individual's personas which are not in our contact
+   * list. */
+  personas = folks_individual_get_personas (individual);
+  iter = gee_iterable_iterator (GEE_ITERABLE (personas));
+  while (gee_iterator_next (iter))
+    {
+      TpfPersona *persona = gee_iterator_get (iter);
+      TpContact *contact;
+      TpConnection *conn;
+
+      if (!TPF_IS_PERSONA (persona))
+        goto next;
+
+      contact = tpf_persona_get_contact (persona);
+      if (contact == NULL)
+        goto next;
+
+      /* be sure to use a not channel specific contact.
+       * TODO: Ideally tp-glib should do this for us (fdo #42702)*/
+      if (EMPATHY_IS_INDIVIDUAL_STORE_CHANNEL (priv->store))
+        {
+          TpChannel *channel;
+          TpChannelGroupFlags flags;
+
+          channel = empathy_individual_store_channel_get_channel (
+              EMPATHY_INDIVIDUAL_STORE_CHANNEL (priv->store));
+
+          flags = tp_channel_group_get_flags (channel);
+          if ((flags & TP_CHANNEL_GROUP_FLAG_CHANNEL_SPECIFIC_HANDLES) != 0)
+            {
+              /* Channel uses channel specific handles (thanks XMPP...) */
+              contact = tp_channel_group_get_contact_owner (channel, contact);
+
+              /* If we don't know the owner, we can't add the contact */
+              if (contact == NULL)
+                goto next;
+            }
+        }
+
+      conn = tp_contact_get_connection (contact);
+      if (conn == NULL)
+        goto next;
+
+      /* No point to try adding a contact if the CM doesn't support it */
+      if (!tp_connection_get_can_change_contact_list (conn))
+        goto next;
+
+      /* Can't add ourself */
+      if (tp_connection_get_self_contact (conn) == contact)
+        goto next;
+
+      if (tp_contact_get_subscribe_state (contact) == TP_SUBSCRIPTION_STATE_YES)
+        goto next;
+
+      g_object_unref (persona);
+      to_add = contact;
+      break;
+
+next:
+      g_object_unref (persona);
+    }
+
+  g_object_unref (iter);
+
+  if (to_add == NULL)
+    return NULL;
+
+  item = gtk_image_menu_item_new_with_mnemonic (_("_Add Contact…"));
+  image = gtk_image_new_from_icon_name (GTK_STOCK_ADD, GTK_ICON_SIZE_MENU);
+  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+
+  g_signal_connect_data (item, "activate",
+      G_CALLBACK (add_menu_item_activated),
+      g_object_ref (to_add), (GClosureNotify) g_object_unref, 0);
+
+  return item;
+}
