@@ -46,7 +46,7 @@
 #include "empathy-individual-view.h"
 #include "empathy-individual-menu.h"
 #include "empathy-individual-store.h"
-#include "empathy-contact-dialogs.h"
+#include "empathy-individual-edit-dialog.h"
 #include "empathy-individual-dialogs.h"
 #include "empathy-images.h"
 #include "empathy-linking-dialog.h"
@@ -74,6 +74,7 @@ typedef struct
 
   gboolean show_offline;
   gboolean show_untrusted;
+  gboolean show_uninteresting;
 
   GtkTreeModelFilter *filter;
   GtkWidget *search_widget;
@@ -114,6 +115,7 @@ enum
   PROP_INDIVIDUAL_FEATURES,
   PROP_SHOW_OFFLINE,
   PROP_SHOW_UNTRUSTED,
+  PROP_SHOW_UNINTERESTING,
 };
 
 /* TODO: re-add DRAG_TYPE_CONTACT_ID, for the case that we're dragging around
@@ -986,7 +988,6 @@ individual_view_key_press_event_cb (EmpathyIndividualView *view,
       g_idle_add (individual_view_popup_menu_idle_cb, data);
     } else if (event->keyval == GDK_KEY_F2) {
         FolksIndividual *individual;
-        EmpathyContact *contact;
 
         g_return_val_if_fail (EMPATHY_IS_INDIVIDUAL_VIEW (view), FALSE);
 
@@ -994,15 +995,9 @@ individual_view_key_press_event_cb (EmpathyIndividualView *view,
         if (individual == NULL)
             return FALSE;
 
-        contact = empathy_contact_dup_from_folks_individual (individual);
-        if (contact == NULL) {
-            g_object_unref (individual);
-            return FALSE;
-        }
-        empathy_contact_edit_dialog_show (contact, NULL);
+        empathy_individual_edit_dialog_show (individual, NULL);
 
         g_object_unref (individual);
-        g_object_unref (contact);
     }
 
   return FALSE;
@@ -1078,12 +1073,12 @@ individual_view_call_activated_cb (EmpathyCellRendererActivatable *cell,
   shell = GTK_MENU_SHELL (menu);
 
   /* audio */
-  item = empathy_individual_audio_call_menu_item_new (individual, NULL);
+  item = empathy_individual_audio_call_menu_item_new (individual);
   gtk_menu_shell_append (shell, item);
   gtk_widget_show (item);
 
   /* video */
-  item = empathy_individual_video_call_menu_item_new (individual, NULL);
+  item = empathy_individual_video_call_menu_item_new (individual);
   gtk_menu_shell_append (shell, item);
   gtk_widget_show (item);
 
@@ -1532,7 +1527,7 @@ expand_idle_foreach_cb (GtkTreeModel *model,
       EMPATHY_INDIVIDUAL_STORE_COL_NAME, &name,
       -1);
 
-  if (is_group == FALSE)
+  if (!is_group)
     {
       g_free (name);
       return FALSE;
@@ -1541,9 +1536,9 @@ expand_idle_foreach_cb (GtkTreeModel *model,
   priv = GET_PRIV (self);
 
   if (g_hash_table_lookup_extended (priv->expand_groups, name, NULL,
-      &should_expand) == TRUE)
+      &should_expand))
     {
-      if (GPOINTER_TO_INT (should_expand) == TRUE)
+      if (GPOINTER_TO_INT (should_expand))
         gtk_tree_view_expand_row (GTK_TREE_VIEW (self), path, FALSE);
       else
         gtk_tree_view_collapse_row (GTK_TREE_VIEW (self), path);
@@ -1622,8 +1617,8 @@ individual_view_row_has_child_toggled_cb (GtkTreeModel *model,
    * gtk_tree_model_filter_refilter (). We add the rows to expand/contract to
    * a hash table, and expand or contract them as appropriate all at once in
    * an idle handler which iterates over all the group rows. */
-  if (g_hash_table_lookup_extended (priv->expand_groups, name, NULL,
-      &will_expand) == FALSE ||
+  if (!g_hash_table_lookup_extended (priv->expand_groups, name, NULL,
+      &will_expand) ||
       GPOINTER_TO_INT (will_expand) != should_expand)
     {
       g_hash_table_insert (priv->expand_groups, g_strdup (name),
@@ -1701,7 +1696,7 @@ individual_view_is_visible_individual (EmpathyIndividualView *self,
   EmpathyLiveSearch *live = EMPATHY_LIVE_SEARCH (priv->search_widget);
   GeeSet *personas;
   GeeIterator *iter;
-  gboolean is_favorite, contains_interesting_persona = FALSE;
+  gboolean is_favorite;
 
   /* Always display individuals having pending events */
   if (event_count > 0)
@@ -1709,32 +1704,38 @@ individual_view_is_visible_individual (EmpathyIndividualView *self,
 
   /* We're only giving the visibility wrt filtering here, not things like
    * presence. */
-  if (priv->show_untrusted == FALSE &&
+  if (!priv->show_untrusted &&
       folks_individual_get_trust_level (individual) == FOLKS_TRUST_LEVEL_NONE)
     {
       return FALSE;
     }
 
-  /* Hide all individuals which consist entirely of uninteresting personas */
-  personas = folks_individual_get_personas (individual);
-  iter = gee_iterable_iterator (GEE_ITERABLE (personas));
-  while (!contains_interesting_persona && gee_iterator_next (iter))
+  if (!priv->show_uninteresting)
     {
-      FolksPersona *persona = gee_iterator_get (iter);
+      gboolean contains_interesting_persona = FALSE;
 
-      if (empathy_folks_persona_is_interesting (persona))
-        contains_interesting_persona = TRUE;
+      /* Hide all individuals which consist entirely of uninteresting
+       * personas */
+      personas = folks_individual_get_personas (individual);
+      iter = gee_iterable_iterator (GEE_ITERABLE (personas));
+      while (!contains_interesting_persona && gee_iterator_next (iter))
+        {
+          FolksPersona *persona = gee_iterator_get (iter);
 
-      g_clear_object (&persona);
+          if (empathy_folks_persona_is_interesting (persona))
+            contains_interesting_persona = TRUE;
+
+          g_clear_object (&persona);
+        }
+      g_clear_object (&iter);
+
+      if (!contains_interesting_persona)
+        return FALSE;
     }
-  g_clear_object (&iter);
-
-  if (contains_interesting_persona == FALSE)
-    return FALSE;
 
   is_favorite = folks_favourite_details_get_is_favourite (
       FOLKS_FAVOURITE_DETAILS (individual));
-  if (is_searching == FALSE) {
+  if (!is_searching) {
     if (is_favorite && is_fake_group &&
         !tp_strdiff (group, EMPATHY_INDIVIDUAL_STORE_FAVORITE))
         /* Always display favorite contacts in the favorite group */
@@ -1813,7 +1814,7 @@ individual_view_filter_visible_func (GtkTreeModel *model,
       g_free (group);
 
       /* FIXME: Work around bgo#626552/bgo#621076 */
-      if (visible == TRUE)
+      if (visible)
         {
           GtkTreePath *path = gtk_tree_model_get_path (model, iter);
           individual_view_verify_group_visibility (self, path);
@@ -1854,7 +1855,7 @@ individual_view_filter_visible_func (GtkTreeModel *model,
       g_free (group);
 
       /* show group if it has at least one visible contact in it */
-      if (visible == TRUE)
+      if (visible)
         return TRUE;
     }
 
@@ -2046,7 +2047,7 @@ individual_view_finalize (GObject *object)
 
   if (priv->expand_groups_idle_handler != 0)
     g_source_remove (priv->expand_groups_idle_handler);
-  g_hash_table_destroy (priv->expand_groups);
+  g_hash_table_unref (priv->expand_groups);
 
   G_OBJECT_CLASS (empathy_individual_view_parent_class)->finalize (object);
 }
@@ -2078,6 +2079,9 @@ individual_view_get_property (GObject *object,
     case PROP_SHOW_UNTRUSTED:
       g_value_set_boolean (value, priv->show_untrusted);
       break;
+    case PROP_SHOW_UNINTERESTING:
+      g_value_set_boolean (value, priv->show_uninteresting);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
       break;
@@ -2112,6 +2116,9 @@ individual_view_set_property (GObject *object,
       empathy_individual_view_set_show_untrusted (view,
           g_value_get_boolean (value));
       break;
+    case PROP_SHOW_UNINTERESTING:
+      empathy_individual_view_set_show_uninteresting (view,
+          g_value_get_boolean (value));
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
       break;
@@ -2198,6 +2205,13 @@ empathy_individual_view_class_init (EmpathyIndividualViewClass *klass)
           "Whether the view should display untrusted individuals; "
           "those who could not be who they say they are.",
           TRUE, G_PARAM_READWRITE));
+  g_object_class_install_property (object_class,
+      PROP_SHOW_UNINTERESTING,
+      g_param_spec_boolean ("show-uninteresting",
+          "Show Uninteresting Individuals",
+          "Whether the view should not filter out individuals using "
+          "empathy_folks_persona_is_interesting.",
+          FALSE, G_PARAM_READWRITE));
 
   g_type_class_add_private (object_class, sizeof (EmpathyIndividualViewPriv));
 }
@@ -2211,6 +2225,7 @@ empathy_individual_view_init (EmpathyIndividualView *view)
   view->priv = priv;
 
   priv->show_untrusted = TRUE;
+  priv->show_uninteresting = FALSE;
 
   /* Get saved group states. */
   empathy_contact_groups_get_all ();
@@ -2452,12 +2467,12 @@ got_avatar (GObject *source_object,
 {
   FolksIndividual *individual = FOLKS_INDIVIDUAL (source_object);
   EmpathyIndividualView *view = user_data;
+  EmpathyIndividualViewPriv *priv = GET_PRIV (view);
   GdkPixbuf *avatar;
   EmpathyIndividualManager *manager;
   gchar *text;
   GtkWindow *parent;
   GeeSet *personas;
-  GeeIterator *iter;
   guint persona_count = 0;
   gboolean can_block;
   GError *error = NULL;
@@ -2476,21 +2491,31 @@ got_avatar (GObject *source_object,
    * so we still display the remove dialog. */
 
   personas = folks_individual_get_personas (individual);
-  iter = gee_iterable_iterator (GEE_ITERABLE (personas));
+
+  if (priv->show_uninteresting)
+    {
+      persona_count = gee_collection_get_size (GEE_COLLECTION (personas));
+    }
+  else
+    {
+      GeeIterator *iter;
+
+      iter = gee_iterable_iterator (GEE_ITERABLE (personas));
+      while (persona_count < 2 && gee_iterator_next (iter))
+        {
+          FolksPersona *persona = gee_iterator_get (iter);
+
+          if (empathy_folks_persona_is_interesting (persona))
+            persona_count++;
+
+          g_clear_object (&persona);
+        }
+      g_clear_object (&iter);
+    }
 
   /* If we have more than one TpfPersona, display a different message
    * ensuring the user knows that *all* of the meta-contacts' personas will
    * be removed. */
-  while (persona_count < 2 && gee_iterator_next (iter))
-    {
-      FolksPersona *persona = gee_iterator_get (iter);
-
-      if (empathy_folks_persona_is_interesting (persona))
-        persona_count++;
-
-      g_clear_object (&persona);
-    }
-  g_clear_object (&iter);
 
   if (persona_count < 2)
     {
@@ -2595,6 +2620,9 @@ empathy_individual_view_get_individual_menu (EmpathyIndividualView *view)
   if (individual == NULL)
     return NULL;
 
+  if (!empathy_folks_individual_contains_contact (individual))
+    goto out;
+
   /* If any of the Individual's personas can be removed, add an option to
    * remove. This will act as a best-effort option. If any Personas cannot be
    * removed from the server, then this option will just be inactive upon
@@ -2615,7 +2643,8 @@ empathy_individual_view_get_individual_menu (EmpathyIndividualView *view)
     }
   g_clear_object (&iter);
 
-  menu = empathy_individual_menu_new (individual, priv->individual_features);
+  menu = empathy_individual_menu_new (individual, priv->individual_features,
+      priv->store);
 
   /* Remove contact */
   if ((priv->view_features &
@@ -2649,6 +2678,7 @@ empathy_individual_view_get_individual_menu (EmpathyIndividualView *view)
   g_signal_connect (menu, "link-contacts-activated",
       (GCallback) individual_menu_link_contacts_activated_cb, view);
 
+out:
   g_object_unref (individual);
 
   return menu;
@@ -2878,3 +2908,19 @@ empathy_individual_view_select_first (EmpathyIndividualView *self)
       gtk_tree_selection_select_iter (selection, &iter);
     }
 }
+
+void
+empathy_individual_view_set_show_uninteresting (EmpathyIndividualView *self,
+    gboolean show_uninteresting)
+{
+  EmpathyIndividualViewPriv *priv;
+
+  g_return_if_fail (EMPATHY_IS_INDIVIDUAL_VIEW (self));
+
+  priv = GET_PRIV (self);
+
+  priv->show_uninteresting = show_uninteresting;
+
+  g_object_notify (G_OBJECT (self), "show-uninteresting");
+  gtk_tree_model_filter_refilter (priv->filter);
+}
