@@ -48,7 +48,6 @@
 
 #include "empathy-client-factory.h"
 #include "empathy-utils.h"
-#include "empathy-contact-manager.h"
 #include "empathy-individual-manager.h"
 #include "empathy-presence-manager.h"
 #include "empathy-request-util.h"
@@ -1163,3 +1162,32 @@ empathy_sasl_channel_supports_mechanism (TpChannel *channel,
 
   return tp_strv_contains (available_mechanisms, mechanism);
 }
+
+FolksIndividual *
+empathy_create_individual_from_tp_contact (TpContact *contact)
+{
+  GeeSet *personas;
+  TpfPersona *persona;
+  FolksIndividual *individual;
+
+  persona = tpf_persona_dup_for_contact (contact);
+  if (persona == NULL)
+    {
+      DEBUG ("Failed to get a persona for %s",
+          tp_contact_get_identifier (contact));
+      return NULL;
+    }
+
+  personas = GEE_SET (
+      gee_hash_set_new (FOLKS_TYPE_PERSONA, g_object_ref, g_object_unref,
+      g_direct_hash, g_direct_equal));
+
+  gee_collection_add (GEE_COLLECTION (personas), persona);
+
+  individual = folks_individual_new (personas);
+
+  g_clear_object (&persona);
+  g_clear_object (&personas);
+
+  return individual;
+}
