@@ -18,6 +18,7 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include <config.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -220,6 +221,13 @@ create_sink (EmpathyGstAudioSink *self)
 
   empathy_call_set_stream_properties (sink, self->priv->echo_cancel);
 
+  /* Set latency (buffering on the PulseAudio side) of 40ms and transfer data
+   * in 10ms chunks */
+  g_object_set (sink,
+      "buffer-time", (gint64) 40000,
+      "latency-time", (gint64) 10000,
+      NULL);
+
   return sink;
 }
 
