@@ -36,7 +36,6 @@
 #include <libempathy-gtk/empathy-call-utils.h>
 
 #include "empathy-call-handler.h"
-#include "src-marshal.h"
 
 #define DEBUG_FLAG EMPATHY_DEBUG_VOIP
 #include <libempathy/empathy-debug.h>
@@ -48,11 +47,13 @@ enum {
   CONFERENCE_ADDED,
   CONFERENCE_REMOVED,
   SRC_PAD_ADDED,
-  SINK_PAD_ADDED,
-  SINK_PAD_REMOVED,
+  CONTENT_ADDED,
+  CONTENT_REMOVED,
   CLOSED,
   CANDIDATES_CHANGED,
   STATE_CHANGED,
+  FRAMERATE_CHANGED,
+  RESOLUTION_CHANGED,
   LAST_SIGNAL
 };
 
@@ -191,13 +192,18 @@ static void
 on_call_state_changed_cb (TpyCallChannel *call,
   TpyCallState state,
   TpyCallFlags flags,
-   const GValueArray *call_state_reason,
+  const GValueArray *call_state_reason,
   GHashTable *call_state_details,
   EmpathyCallHandler *handler)
 {
   EmpathyCallHandlerPriv *priv = handler->priv;
+  gchar *dbus_reason;
+  guint actor, reason;
 
-  g_signal_emit (handler, signals[STATE_CHANGED], 0, state);
+  tp_value_array_unpack ((GValueArray *) call_state_reason, 3,
+      &actor, &reason, &dbus_reason);
+
+  g_signal_emit (handler, signals[STATE_CHANGED], 0, state, dbus_reason);
 
   if (state == TPY_CALL_STATE_ENDED)
     {
@@ -457,56 +463,69 @@ empathy_call_handler_class_init (EmpathyCallHandlerClass *klass)
   signals[CONFERENCE_ADDED] =
     g_signal_new ("conference-added", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST, 0, NULL, NULL,
-      g_cclosure_marshal_VOID__OBJECT,
+      g_cclosure_marshal_generic,
       G_TYPE_NONE,
       1, FS_TYPE_CONFERENCE);
 
   signals[CONFERENCE_REMOVED] =
     g_signal_new ("conference-removed", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST, 0, NULL, NULL,
-      g_cclosure_marshal_VOID__OBJECT,
+      g_cclosure_marshal_generic,
       G_TYPE_NONE,
       1, FS_TYPE_CONFERENCE);
 
   signals[SRC_PAD_ADDED] =
     g_signal_new ("src-pad-added", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST, 0, NULL, NULL,
-      _src_marshal_BOOLEAN__OBJECT_UINT,
+      g_cclosure_marshal_generic,
       G_TYPE_BOOLEAN,
-      2, GST_TYPE_PAD, G_TYPE_UINT);
+      2, TF_TYPE_CONTENT, GST_TYPE_PAD);
 
-  signals[SINK_PAD_ADDED] =
-    g_signal_new ("sink-pad-added", G_TYPE_FROM_CLASS (klass),
+  signals[CONTENT_ADDED] =
+    g_signal_new ("content-added", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST, 0, NULL, NULL,
-      _src_marshal_BOOLEAN__OBJECT_UINT,
+      g_cclosure_marshal_generic,
       G_TYPE_BOOLEAN,
-      2, GST_TYPE_PAD, G_TYPE_UINT);
+      1, TF_TYPE_CONTENT);
 
-  signals[SINK_PAD_REMOVED] =
-    g_signal_new ("sink-pad-removed", G_TYPE_FROM_CLASS (klass),
+  signals[CONTENT_REMOVED] =
+    g_signal_new ("content-removed", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST, 0, NULL, NULL,
-      _src_marshal_BOOLEAN__OBJECT_UINT,
+      g_cclosure_marshal_generic,
       G_TYPE_BOOLEAN,
-      2, GST_TYPE_PAD, G_TYPE_UINT);
+      1, TF_TYPE_CONTENT);
 
   signals[CLOSED] =
     g_signal_new ("closed", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST, 0, NULL, NULL,
-      g_cclosure_marshal_VOID__VOID,
+      g_cclosure_marshal_generic,
       G_TYPE_NONE,
       0);
 
   signals[CANDIDATES_CHANGED] =
     g_signal_new ("candidates-changed", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST, 0, NULL, NULL,
-      g_cclosure_marshal_VOID__UINT,
+      g_cclosure_marshal_generic,
       G_TYPE_NONE, 1, G_TYPE_UINT);
 
   signals[STATE_CHANGED] =
     g_signal_new ("state-changed", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST, 0, NULL, NULL,
-      g_cclosure_marshal_VOID__UINT,
+      g_cclosure_marshal_generic,
+      G_TYPE_NONE, 2, G_TYPE_UINT, G_TYPE_STRING);
+
+  signals[FRAMERATE_CHANGED] =
+    g_signal_new ("framerate-changed", G_TYPE_FROM_CLASS (klass),
+      G_SIGNAL_RUN_LAST, 0, NULL, NULL,
+      g_cclosure_marshal_generic,
       G_TYPE_NONE, 1, G_TYPE_UINT);
+
+  signals[RESOLUTION_CHANGED] =
+    g_signal_new ("resolution-changed", G_TYPE_FROM_CLASS (klass),
+      G_SIGNAL_RUN_LAST, 0, NULL, NULL,
+      g_cclosure_marshal_generic,
+      G_TYPE_NONE,
+      2, G_TYPE_UINT, G_TYPE_UINT);
 }
 
 EmpathyCallHandler *
@@ -744,25 +763,46 @@ on_tf_content_src_pad_added_cb (TfContent *content,
   FsCodec *codec,
   EmpathyCallHandler *handler)
 {
-  guint media_type;
   gboolean retval;
 
-  g_object_get (content, "media-type", &media_type, NULL);
-
   g_signal_emit (G_OBJECT (handler), signals[SRC_PAD_ADDED], 0,
-      pad, media_type, &retval);
+      content, pad, &retval);
 
   if (!retval)
     g_idle_add (src_pad_added_error_idle, g_object_ref (content));
 }
 
+static void
+on_tf_content_framerate_changed (TfContent *content,
+  GParamSpec *spec,
+  EmpathyCallHandler *handler)
+{
+  guint framerate;
+
+  g_object_get (content, "framerate", &framerate, NULL);
+
+  if (framerate != 0)
+    g_signal_emit (G_OBJECT (handler), signals[FRAMERATE_CHANGED], 0,
+        framerate);
+}
+
+static void
+on_tf_content_resolution_changed (TfContent *content,
+   guint width,
+   guint height,
+   EmpathyCallHandler *handler)
+{
+  if (width > 0 && height > 0)
+    g_signal_emit (G_OBJECT (handler), signals[RESOLUTION_CHANGED], 0,
+        width, height);
+}
+
 static void
 on_tf_channel_content_added_cb (TfChannel *tfchannel,
   TfContent *content,
   EmpathyCallHandler *handler)
 {
   FsMediaType mtype;
-  GstPad *spad;
   FsSession *session;
 //  FsStream *fs_stream;
   FsCodec *codec;
@@ -778,11 +818,8 @@ on_tf_channel_content_added_cb (TfChannel *tfchannel,
       G_CALLBACK (on_tf_content_stop_sending_cb), handler);
 #endif
 
-  g_object_get (content, "media-type", &mtype,
-    "sink-pad", &spad, NULL);
-
-  g_signal_emit (G_OBJECT (handler), signals[SINK_PAD_ADDED], 0,
-      spad, mtype, &retval);
+  g_signal_emit (G_OBJECT (handler), signals[CONTENT_ADDED], 0,
+    content, &retval);
 
  if (!retval)
       tf_content_error (content, 0 /* FIXME */,
@@ -808,7 +845,34 @@ on_tf_channel_content_added_cb (TfChannel *tfchannel,
  tp_clear_object (&fs_stream);
 */
 
- gst_object_unref (spad);
+  g_object_get (content, "media-type", &mtype, NULL);
+
+ if (mtype == FS_MEDIA_TYPE_VIDEO)
+   {
+     guint framerate, width, height;
+
+     g_signal_connect (content, "notify::framerate",
+         G_CALLBACK (on_tf_content_framerate_changed),
+         handler);
+
+     g_signal_connect (content, "resolution-changed",
+         G_CALLBACK (on_tf_content_resolution_changed),
+         handler);
+
+     g_object_get (content,
+         "framerate", &framerate,
+         "width", &width,
+         "height", &height,
+         NULL);
+
+     if (framerate > 0)
+       g_signal_emit (G_OBJECT (handler), signals[FRAMERATE_CHANGED], 0,
+           framerate);
+
+     if (width > 0 && height > 0)
+       g_signal_emit (G_OBJECT (handler), signals[RESOLUTION_CHANGED], 0,
+           width, height);
+   }
 }
 
 static void
@@ -816,17 +880,12 @@ on_tf_channel_content_removed_cb (TfChannel *tfchannel,
   TfContent *content,
   EmpathyCallHandler *handler)
 {
-  FsMediaType mtype;
-  GstPad *spad;
   gboolean retval;
 
   DEBUG ("removing content");
 
-  g_object_get (content, "media-type", &mtype,
-    "sink-pad", &spad, NULL);
-
-  g_signal_emit (G_OBJECT (handler), signals[SINK_PAD_REMOVED], 0,
-      spad, mtype, &retval);
+  g_signal_emit (G_OBJECT (handler), signals[CONTENT_REMOVED], 0,
+      content, &retval);
 
   if (!retval)
     {
